#!/usr/bin/env python
# Copyright (c) 2013 WindLand
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
# of the Software, and to permit persons to whom the Software is furnished to do
# so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import time
__all__ = ['HIGH', 'LOW', 'output', 'input', 'usleep']

_GPIO_PINS = ('pb10', 'pd0', 'pd14', 'pd2', 'pd25', 'pd6', 'pe4', 'pg0', 'pg4', 'ph14',
            'pi13','pi9', 'pb11', 'pd1', 'pd15', 'pd20', 'pd26', 'pd7', 'pe5', 'pg1', 'pg5',
            'ph15', 'pi4', 'pb13', 'pd10', 'pd16', 'pd21', 'pd27', 'pd8', 'pe6', 'pg10',
            'pg6', 'ph7', 'pi5', 'pb18', 'pd11', 'pd17', 'pd22', 'pd3', 'pd9', 'pe7',
            'pg11', 'pg7', 'pi10', 'pi6', 'pb19', 'pd12', 'pd18', 'pd23', 'pd4', 'pe10',
            'pe8', 'pg2', 'pg8', 'pi11', 'pi7', 'pb2', 'pd13', 'pd19', 'pd24', 'pd5',
            'pe11', 'pe9', 'pg3', 'pg9', 'pi12', 'pi8')

_PIN_FD_PATH = '/sys/devices/virtual/misc/sun4i-gpio/pin/%s'

HIGH = 1
LOW = 0

class InvalidChannelException(Exception):
    """The channel sent is invalid on a Cubieboard"""
    pass

def _GetValidId(channel):
    if channel in _GPIO_PINS:
        return channel
    else:
        raise InvalidChannelException

def usleep(ms):
    time.sleep(ms/1000000.0)

def output(channel, value):
    """Write to a GPIO channel"""
    id = _GetValidId(channel)
    with open(_PIN_FD_PATH % id, 'w') as f:
        f.write('1' if value else '0')

def input(channel):
    """Read from a GPIO channel"""
    id = _GetValidId(channel)
    with open(_PIN_FD_PATH % id, 'r') as f:
        return f.read(1) == '1'

if __name__ == '__main__':
    output('pd0', HIGH)
    output('pd0', LOW)
