#define _LARGEFILE64_SOURCE
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>

#define STAGE1_OFFSET     (8  * 1024)
#define STAGE2_OFFSET     (32 * 1024)
#define STAGE1_SIZE_LIMIT (24  * 1024)
#define STAGE2_SIZE_LIMIT (992 * 1024)

#define STAGE1_FLAG   0
#define STAGE2_FLAG   1
#define UNDEFINE_FLAG 2


int main(int argc, char *argv[])
{
	unsigned int file_size;
	off64_t sd_size;
	int flag = STAGE1_FLAG;
	int temp = 0;
	int ret = -1;
	int ifd;
	int ofd;
	char input_file[STAGE2_SIZE_LIMIT] = {0};

	if((argc < 3) || (strncmp(argv[2], "/dev/", 5))){
		fprintf(stderr, "Usage: sd_write  <Input file>  <Output file>\n");
		return -1;
	}

	ofd = open(argv[2], O_WRONLY | O_LARGEFILE);
	if(ofd == -1){
		perror(argv[2]);
		goto err0;
	}
	
	ifd = open(argv[1], O_RDONLY);
	if(ifd == -1){
		perror(argv[1]);
		goto err1;
	}
	 
	if(argc == 4){
		temp = atoi(argv[3]);
		if( 1 == temp ){
			flag = STAGE1_FLAG;
		}
		else if( 2 == temp ){
			flag = STAGE2_FLAG;
		}
		else{
			flag = UNDEFINE_FLAG;
		}
	}
	else{
		flag = UNDEFINE_FLAG;
	}
/*************************************************************************/
	if((file_size = lseek(ifd, 0, SEEK_END)) == -1){
		perror("lseek()");
		goto err2;
	}

	if(file_size == 0){
		fprintf(stderr, "Error: File size == 0\n");
		goto err2;
	}

	if(lseek(ifd, 0, SEEK_SET) == -1){
		perror("lseek()");
		goto err2;
	}

	if( flag == STAGE1_FLAG ){
		if( file_size > STAGE1_SIZE_LIMIT ){
			fprintf(stderr, "Error: File size > %uK.\n", STAGE1_SIZE_LIMIT / 1024);
			goto err2;
		}
	}
	else if( flag == STAGE2_FLAG ){
		if( file_size > STAGE2_SIZE_LIMIT ){
			fprintf(stderr, "Error: File size > %uK.\n", STAGE2_SIZE_LIMIT / 1024);
			goto err2;
		}
	
	}
	else{
		if( file_size < STAGE1_SIZE_LIMIT ){
			flag = STAGE1_FLAG;
			
		}
		else if( file_size < STAGE2_SIZE_LIMIT ){
			flag = STAGE2_FLAG;
		}
		fprintf(stdout, "file_size = %uK flag = %u .\n", file_size /1024, flag);
	}


	if(read( ifd, input_file, file_size ) == -1){
		perror("read()");
		goto err2;
	}

/*************************************************************************/

	if((sd_size = lseek64(ofd, 0, SEEK_END)) == -1){
		perror("lseek64()");
		goto err2;
	}

	if( sd_size < 0x100000 ){
		fprintf(stderr, "Error: SD card < 1M.\n");
		goto err2;
	}

/*************************************************************************/
		
	if(lseek64(ofd, 0, SEEK_SET) == -1){
		perror("lseek64()");
		goto err2;
	}

	if(flag == STAGE1_FLAG){
		if( lseek64(ofd, STAGE1_OFFSET, SEEK_SET) == -1){
			perror("lseek64()");
			goto err2;
		}
		fprintf(stdout, "write in stage1, offset = %uK.\n", STAGE1_OFFSET / 1024);
	}
	else{
		if(lseek64(ofd, STAGE2_OFFSET, SEEK_SET) == -1){
			perror("lseek64()");
			goto err2;
		}
		fprintf(stdout, "write in stage2, offset = %uK.\n", STAGE2_OFFSET / 1024);
	}


	if(write(ofd, input_file, file_size) != file_size){
		fprintf(stderr, "write failed.\n");
		goto err2;
	}
	
/*************************************************************************/
	fsync(ofd);

	printf(" Done.\n");
	ret = 0;

err2:
	close(ifd);

err1:
	close(ofd);

err0:
	return ret;
}


